@@ -3,20 +3,15 @@ package DBICTest::Util::LeakTracer;
 use warnings;
 use strict;
 
-use ANFANG;
 use Carp;
 use Scalar::Util qw(isweak weaken blessed reftype);
-use DBIx::Class::_Util qw(refcount hrefaddr refdesc dump_value visit_namespaces);
-use DBICTest::RunMode;
+use DBIx::Class::_Util qw(refcount hrefaddr);
+use DBIx::Class::Optional::Dependencies;
+use Data::Dumper::Concise;
 use DBICTest::Util 'stacktrace';
 use constant {
-  CV_TRACING => !!(
-    !DBICTest::RunMode->is_plain
-      &&
-    require DBIx::Class::Optional::Dependencies
-      &&
-    DBIx::Class::Optional::Dependencies->req_ok_for ('test_leaks_heavy')
-  ),
+  CV_TRACING => DBIx::Class::Optional::Dependencies->req_ok_for ('test_leaks_heavy'),
+  SKIP_SCALAR_REFS => ( $] > 5.017 ) ? 1 : 0,
 };
 
 use base 'Exporter';
@@ -26,6 +21,15 @@ my $refs_traced = 0;
 my $leaks_found = 0;
 my %reg_of_regs;
 
+# so we don't trigger stringification
+sub _describe_ref {
+  sprintf '%s%s(%s)',
+    (defined blessed $_[0]) ? blessed($_[0]) . '=' : '',
+    reftype $_[0],
+    hrefaddr $_[0],
+  ;
+}
+
 sub populate_weakregistry {
   my ($weak_registry, $target, $note) = @_;
 
@@ -48,26 +52,20 @@ sub populate_weakregistry {
       for keys %$reg;
   }
 
-  return $target if (
-    DBIx::Class::_ENV_::BROKEN_WEAK_SCALARREF_VALUES
-      and
-    ref $target eq 'SCALAR'
-  );
+  # FIXME/INVESTIGATE - something fishy is going on with refs to plain
+  # strings, perhaps something to do with the CoW work etc...
+  return $target if SKIP_SCALAR_REFS and reftype($target) eq 'SCALAR';
 
   if (! defined $weak_registry->{$refaddr}{weakref}) {
-
-    # replace slot entirely
     $weak_registry->{$refaddr} = {
       stacktrace => stacktrace(1),
       weakref => $target,
     };
-
     weaken( $weak_registry->{$refaddr}{weakref} );
-
     $refs_traced++;
   }
 
-  my $desc = refdesc $target;
+  my $desc = _describe_ref($target);
   $weak_registry->{$refaddr}{slot_names}{$desc} = 1;
   if ($note) {
     $note =~ s/\s*\Q$desc\E\s*//g;
@@ -78,7 +76,7 @@ sub populate_weakregistry {
 }
 
 # Regenerate the slots names on a thread spawn
-sub DBICTest::__LeakTracer_iThreads_handler__::CLONE {
+sub CLONE {
   my @individual_regs = grep { scalar keys %{$_||{}} } values %reg_of_regs;
   %reg_of_regs = ();
 
@@ -102,11 +100,6 @@ sub DBICTest::__LeakTracer_iThreads_handler__::CLONE {
       $reg->{$new_addr} = $slot_info;
     }
   }
-
-  # Dummy NEXTSTATE ensuring the all temporaries on the stack are garbage
-  # collected before leaving this scope. Depending on the code above, this
-  # may very well be just a preventive measure guarding future modifications
-  undef;
 }
 
 sub visit_refs {
@@ -139,7 +132,6 @@ sub visit_refs {
 
     my $type = reftype $r;
 
-    local $SIG{__DIE__} if $SIG{__DIE__};
     local $@;
     eval {
       if ($type eq 'HASH') {
@@ -158,29 +150,46 @@ sub visit_refs {
       elsif (CV_TRACING and $type eq 'CODE') {
         $visited_cnt += visit_refs({ %$args, refs => [ map {
           ( !isweak($_) ) ? $_ : ()
-        } values %{ scalar PadWalker::closed_over($r) } ] }); # scalar due to RT#92269
+        } scalar PadWalker::closed_over($r) ] }); # scalar due to RT#92269
       }
       1;
-    } or (
-      # this is some bizarre old DBI autosplit thing, no point mentioning it
-      $@ !~ m{ ^Can't \s locate \s (?:
-        auto/DBI/FIRSTKEY.al
-          |
-        \Qobject method "FIRSTKEY" via package "DBI"\E
-      )}x
-        and
-      warn "Could not descend into @{[ refdesc $r ]}: $@\n"
-    );
+    } or warn "Could not descend into @{[ _describe_ref($r) ]}: $@\n";
   }
   $visited_cnt;
 }
 
+sub visit_namespaces {
+  my $args = { (ref $_[0]) ? %{$_[0]} : @_ };
+
+  my $visited = 1;
+
+  $args->{package} ||= '::';
+  $args->{package} = '::' if $args->{package} eq 'main';
+
+  if ( $args->{action}->($args->{package}) ) {
+
+    my $base = $args->{package};
+    $base = '' if $base eq '::';
+
+
+    $visited += visit_namespaces({ %$args, package => $_ }) for map
+      { $_ =~ /(.+?)::$/ && "${base}::$1" }
+      grep
+        { $_ =~ /(?<!^main)::$/ }
+        do {  no strict 'refs'; keys %{ $base . '::'} }
+  }
+
+  return $visited;
+}
+
 # compiles a list of addresses stored as globals (possibly even catching
 # class data in the form of method closures), so we can skip them further on
 sub symtable_referenced_addresses {
 
   my $refs_per_pkg;
 
+  my $dummy_addresslist;
+
   my $seen_refs = {};
   visit_namespaces(
     action => sub {
@@ -188,32 +197,41 @@ sub symtable_referenced_addresses {
       no strict 'refs';
 
       my $pkg = shift;
+      $pkg = '' if $pkg eq '::';
+      $pkg .= '::';
 
       # the unless regex at the end skips some dangerous namespaces outright
       # (but does not prevent descent)
       $refs_per_pkg->{$pkg} += visit_refs (
         seen_refs => $seen_refs,
 
-        action => sub { 1 },
+        # FIXME FIXME FIXME
+        # This is so damn odd - if we feed a constsub {1} (or in fact almost
+        # anything other than the actionsub below, any scalarref will show
+        # up as a leak, trapped by... something...
+        # Ideally we should be able to const this to sub{1} and just return
+        # $seen_refs (in fact it is identical to the dummy list at the end of
+        # a run here). Alas this doesn't seem to work, so punt for now...
+        action => sub { ++$dummy_addresslist->{ hrefaddr $_[0] } },
 
         refs => [ map { my $sym = $_;
-          # *{"${pkg}::$sym"}{CODE} won't simply work - MRO-cached CVs are invisible there
-          ( CV_TRACING ? Class::MethodCache::get_cv("${pkg}::$sym") : () ),
+          # *{"$pkg$sym"}{CODE} won't simply work - MRO-cached CVs are invisible there
+          ( CV_TRACING ? Class::MethodCache::get_cv("${pkg}$sym") : () ),
 
-          ( defined *{"${pkg}::$sym"}{SCALAR} and length ref ${"${pkg}::$sym"} and ! isweak( ${"${pkg}::$sym"} ) )
-            ? ${"${pkg}::$sym"} : ()
+          ( defined *{"$pkg$sym"}{SCALAR} and length ref ${"$pkg$sym"} and ! isweak( ${"$pkg$sym"} ) )
+            ? ${"$pkg$sym"} : ()
           ,
 
           ( map {
-            ( defined *{"${pkg}::$sym"}{$_} and ! isweak(defined *{"${pkg}::$sym"}{$_}) )
-              ? *{"${pkg}::$sym"}{$_}
+            ( defined *{"$pkg$sym"}{$_} and ! isweak(defined *{"$pkg$sym"}{$_}) )
+              ? *{"$pkg$sym"}{$_}
               : ()
           } qw(HASH ARRAY IO GLOB) ),
 
-        } keys %{"${pkg}::"} ],
-      ) unless $pkg =~ /^ (?:
+        } keys %$pkg ],
+      ) unless $pkg =~ /^ :: (?:
         DB | next | B | .+? ::::ISA (?: ::CACHE ) | Class::C3
-      ) $/x;
+      ) :: $/x;
     }
   );
 
@@ -234,7 +252,7 @@ sub assert_empty_weakregistry {
   # in case we hooked bless any extra object creation will wreak
   # havoc during the assert phase
   local *CORE::GLOBAL::bless;
-  *CORE::GLOBAL::bless = sub { CORE::bless( $_[0], (@_ > 1) ? $_[1] : CORE::caller() ) };
+  *CORE::GLOBAL::bless = sub { CORE::bless( $_[0], (@_ > 1) ? $_[1] : caller() ) };
 
   croak 'Expecting a registry hashref' unless ref $weak_registry eq 'HASH';
 
@@ -257,32 +275,19 @@ sub assert_empty_weakregistry {
       if defined $weak_registry->{$addr}{weakref} and ! isweak( $weak_registry->{$addr}{weakref} );
   }
 
-  # the symtable walk is very expensive
-  # if we are $quiet (running in an END block) we do not really need to be
-  # that thorough - can get by with our own registry
-  delete $weak_registry->{$_} for $quiet
-    ? do {
-      my $refs = {};
-      visit_refs (
-        # only look at the closed over stuffs
-        refs => [ values %DBIx::Class::_Util::refs_closed_over_by_quote_sub_installed_crefs ],
-        seen_refs => $refs,
-        action => sub { 1 },
-      );
-      keys %$refs;
-    }
-    : (
-      # full sumtable walk, starting from ::
-      keys %{ symtable_referenced_addresses() }
-    )
-  ;
+  # the walk is very expensive - if we are $quiet (running in an END block)
+  # we do not really need to be too thorough
+  unless ($quiet) {
+    delete $weak_registry->{$_} for keys %{ symtable_referenced_addresses() };
+  }
+
 
   for my $addr (sort { $weak_registry->{$a}{display_name} cmp $weak_registry->{$b}{display_name} } keys %$weak_registry) {
 
     next if ! defined $weak_registry->{$addr}{weakref};
 
     $leaks_found++ unless $tb->in_todo;
-    $tb->ok (0, "Expected garbage collection of $weak_registry->{$addr}{display_name}");
+    $tb->ok (0, "Leaked $weak_registry->{$addr}{display_name}");
 
     my $diag = do {
       local $Data::Dumper::Maxdepth = 1;
@@ -293,7 +298,7 @@ sub assert_empty_weakregistry {
           ref($weak_registry->{$addr}{weakref}) eq 'CODE'
             and
           B::svref_2object($weak_registry->{$addr}{weakref})->XSUB
-        ) ? '__XSUB__' : dump_value $weak_registry->{$addr}{weakref}
+        ) ? '__XSUB__' : Dumper( $weak_registry->{$addr}{weakref} )
       ;
     };
 
@@ -320,27 +325,20 @@ sub assert_empty_weakregistry {
 #      Devel::MAT::Dumper::dumpfh( $fh );
 #      close ($fh) or die $!;
 #
-#      require POSIX;
+#      use POSIX;
 #      POSIX::_exit(1);
 #    }
   }
 
   if (! $quiet and !$leaks_found and ! $tb->in_todo) {
-    $tb->ok(1, sprintf "No leaks found at %s line %d", (CORE::caller())[1,2] );
+    $tb->ok(1, sprintf "No leaks found at %s line %d", (caller())[1,2] );
   }
 }
 
 END {
-  if (
-    $INC{'Test/Builder.pm'}
-      and
-    my $tb = do {
-      local $@;
-      my $t = eval { Test::Builder->new }
-        or warn "Test::Builder->new failed:\n$@\n";
-      $t;
-    }
-  ) {
+  if ($INC{'Test/Builder.pm'}) {
+    my $tb = Test::Builder->new;
+
     # we check for test passage - a leak may be a part of a TODO
     if ($leaks_found and !$tb->is_passing) {
 
@@ -354,24 +352,6 @@ END {
     else {
       $tb->note("Auto checked $refs_traced references for leaks - none detected");
     }
-
-    # also while we are here and not in plain runmode: make sure we never
-    # loaded any of the strictures XS bullshit (it's a leak in a sense)
-    unless (
-      $ENV{MOO_FATAL_WARNINGS}
-        or
-      # FIXME - SQLT loads strictures explicitly, /facedesk
-      # remove this INC check when 0fb58589 and 45287c815 are rectified
-      $INC{'SQL/Translator.pm'}
-        or
-      DBICTest::RunMode->is_plain
-    ) {
-      for (qw(indirect multidimensional bareword::filehandles)) {
-        exists $INC{ Module::Runtime::module_notional_filename($_) }
-          and
-        $tb->ok(0, "$_ load should not have been attempted!!!" )
-      }
-    }
   }
 }
 
